#include "tcpip.h"

PLUGIN_STD_DEFS();

/* Mask to determine fragments */
unsigned short ipfmask=__constant_htons(IP_MF|IP_OFFMASK);

proc_preproc_activate preproc_activate;
proc_serial_number serial_number;
proc_matcher_find matcher_find;
proc_args_parse args_parse;

/* Detection stuff */
proc_detect_add_sig detect_add_sig;
proc_detect_free_sig detect_free_sig;
proc_detect_set detect_set;
proc_detect detect;
proc_alert alert;
proc_dispatch dispatch;

struct alert *cur_alert;
unsigned int alert_depth;
unsigned int cur_depth;

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	proc_preproc_add preproc_add;
	proc_decode_add decode_add;
	proc_generator_add generator_add;
	unsigned int ok=0;
	
	plugin_check(in, out);
	
	PLUGIN_ID("decode.tcpip", "The Internet Protocol");
	PLUGIN_VERSION(2, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(decode_add=in->import("decode.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(preproc_activate=in->import("preproc.activate")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(args_parse=in->import("args.parse")) ) {
		return PLUGIN_ERR_OBJECT;
	}
	
	if ( !(matcher_find=in->import("matcher.find")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(generator_add=in->import("generator.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}
	
	if ( !(preproc_add=in->import("preproc.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(serial_number=in->import("serial_number")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(alert=in->import("alert")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(dispatch=in->import("dispatch")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(detect_add_sig=in->import("detect.add_sig")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(detect_free_sig=in->import("detect.free_sig")) ) {		
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(detect_set=in->import("detect.set")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(detect=in->import("detect")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=generator_add(&ip_gen);
	ok+=generator_add(&dip_gen);
	ok+=generator_add(&tcp_gen);
	ok+=generator_add(&udp_gen);
	ok+=generator_add(&icmp_gen);
	ok+=generator_add(&ipfrag_gen);
	ok+=generator_add(&tcpstream_gen);

	ok+=decode_add(&ipv4_p, ipv4_r);
	ok+=decode_add(&icmp_p, icmp_r);
	ok+=decode_add(&tcp_p, tcp_r);
	ok+=decode_add(&udp_p, udp_r);

	ok+=preproc_add("ipfrag", ipfrag_init);
	ok+=preproc_add("tcpstream", tcpstream_init);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	if ( use_ipfrag ) ipfrag_free();
	if ( tcp_stateful ) tcpstream_free();
	return PLUGIN_ERR_OK;
}
