#ifndef __DECODE_HEADER_INCLUDED__
#define __DECODE_HEADER_INCLUDED__

#define PROTO_LEN 15

typedef unsigned int proto_id_t;

struct proto_child {
	struct proto_child	*next;
	proto_id_t		id;
	struct proto		*proto;
};

#define proto_request(x, y) {x, y}
#define null_request() {NULL, 0}
struct proto_req {
	char			*name;
	proto_id_t		id;
};

#ifndef __PLUGIN__
/* temporary structure used internally */
struct preq_list {
	struct preq_list	*next;
	struct proto		*proto;
	struct proto_req	*arr;
};
#endif

/* decode a packet */
typedef void (*proc_decode)(struct packet *);

/* print it in human-readable form */
typedef int  (*proc_dprint)(struct layer*, char *, int);

/* signature engine */
typedef int  (*proc_sig_add)(struct rule *);
typedef void (*proc_sig_match)(struct packet *, unsigned int);

/* serialize/deserialize session data for file storage */
typedef int  (*proc_serialize)(struct packet*, unsigned int, char *, int);
typedef void *(*proc_deserialize)(struct packet*, unsigned int, char *, int);
typedef void (*proc_session_free)(void *);

/* struct proto, and some initializer macros */
#define init_proto(x,y,z)        { .name=x, .decode=y, .print=z, }
#define init_proto2(x,y,z,a,b)   { .name=x, .decode=y, .print=z, .sig_match=a, .sig_add=b, }
#define null_proto()             { /* all-zero by default */ }
struct proto {
	struct proto		*next;
	struct proto_child	*children;

	/* Obvious stuff */
	const char		*name;
	proc_decode		decode;

	/* print out a one line summary given a header */
	proc_dprint		print;

	/* signature engine */
	proc_sig_match		sig_match;
	proc_sig_add		sig_add;

	/* session/stream oriented protocols */
	proc_session_free	free;
	proc_serialize		serialize;
	proc_deserialize	deserialize;
#define PSF_REQUEST		(1<<0)
#define PSF_REPLY		(1<<1)
	u_int32_t		sflags;

	/* preprocessors */
	struct pp_active	*preproc;
	unsigned int		num_active;
	unsigned int		max_active;
};

#ifndef __PLUGIN__
void decode_init(void);
int decode_resolve(struct proto_req *, struct proto *);
int decode_resolve_all(void);
#endif

#ifndef __PLUGIN__
int decode_add(struct proto *, struct proto_req *);
struct proto *decode_proto(char *);
struct proto *decode_subproto(char *, unsigned int);
#else
typedef int(*proc_decode_add)(struct proto *, struct proto_req *);
typedef struct proto *(*proc_decode_proto)(char *);
typedef struct proto *(*proc_decode_subproto)(char *, unsigned int);
#endif

extern struct proto pcap_dlt_p;

#endif /* __DECODE_HEADER_INCLUDED__ */
