#ifndef __PLUGIN_HEADER_INCLUDED__
#define __PLUGIN_HEADER_INCLUDED__

/* requires: sys/types.h */
/* requires: firestorm.h */

#define PLUGIN_ERR_OK		0 /* success */
#define PLUGIN_ERR_FAIL		1 /* some internal failure */
#define PLUGIN_ERR_PARAM	2 /* bad paramaeter */
#define PLUGIN_ERR_NOMEM	3 /* ran out of memory during initialisation */
#define PLUGIN_ERR_SIZE		4 /* wrong sized object */
#define PLUGIN_ERR_OBJECT	5 /* couldn't load an object */

#define _PLUGIN_INIT	firestorm_plugin_init2
#define PLUGIN_INIT	"firestorm_plugin_init2"

#define _PLUGIN_UNLOAD	firestorm_plugin_unload2
#define PLUGIN_UNLOAD	"firestorm_plugin_unload2"

typedef void *(*proc_plugin_import)(const char *);

/* Host sends this to plugin */
struct plugin_in {
	size_t			size;
	proc_plugin_import	import;
};

/* Plugin sends this to host */
struct plugin_out {
	size_t		size;
	const char	*name;
	const char	*desc;
	unsigned int	ver_major;
	unsigned int	ver_minor;
	const char	*author_name;
	const char	*author_email;
	const char	*license;
};

typedef int (*proc_plugin_init)(struct plugin_in *, struct plugin_out *);
typedef int (*proc_plugin_unload)(int);

/* Plugin helper macros */
#ifdef __PLUGIN__

/* Standard definitions for plugins */
#define PLUGIN_STD_DEFS() \
	proc_mesg mesg;

/* And the declarations */
extern proc_mesg mesg;

/* check parameters in init() function */
#define plugin_check(x, y) \
	do { \
		if ( !x || !y ) \
			return PLUGIN_ERR_PARAM; \
		if ( x->size != sizeof(struct plugin_in) ) \
			return PLUGIN_ERR_SIZE; \
		if ( y->size != sizeof(struct plugin_out) ) \
			return PLUGIN_ERR_SIZE; \
		if ( !x->import ) return PLUGIN_ERR_PARAM; \
		if ( !(mesg=in->import("mesg")) ) return PLUGIN_ERR_OBJECT; \
	} while (0);

#define PLUGIN_ID(x, y) out->name=x; out->desc=y;
#define PLUGIN_VERSION(x, y) out->ver_major=x; out->ver_minor=y;
#define PLUGIN_AUTHOR(x, y) out->author_name=x; out->author_email=y;

/* This is redundant because plugins can only
 * be GPL since they are a derived work of firestorm
 * however it may be useful if you have proprietary
 * plugins that YOU DO NOT DISTRIBUTE */
#define PLUGIN_LICENSE(x) out->license=x;

#endif

#endif /* __PLUGIN_HEADER_INCLUDED__ */
