#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <netinet/in.h>

#include <firestorm.h>
#include <packet.h>
#include <generator.h>
#include <alert.h>
#include <signature.h>
#include <matcher.h>
#include <template.h>
#include <plugin.h>

PLUGIN_STD_DEFS();

proc_template template_shortrange;

int udp_src_match(struct packet *pkt, void *priv, unsigned int l, int n)
{
	struct shortrange *p=(struct shortrange *)priv;
	u_int16_t port=ntohs(pkt->layer[l+1].h.udp->sport);
	return n^(port>=p->min && port<=p->max);
}

int udp_dst_match(struct packet *pkt, void *priv, unsigned int l, int n)
{
	struct shortrange *p=(struct shortrange *)priv;
	u_int16_t port=ntohs(pkt->layer[l+1].h.udp->dport);
	return n^(port>=p->min && port<=p->max);
}

proc_match_match udp_port_validate(proc_match_match fn, char *args, void **priv, struct criteria *m)
{
	if ( !template_shortrange(args, priv) ) return NULL;
	return fn;
}

proc_match_match udp_src_validate(char *args, void **priv,
	struct criteria *m, u_int32_t *c)
{
	return udp_port_validate(udp_src_match, args, priv, m);
}

proc_match_match udp_dst_validate(char *args, void **priv,
	struct criteria *m, u_int32_t *c)
{
	return udp_port_validate(udp_dst_match, args, priv, m);
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	int ok=0;
	proc_matcher_add matcher_add;
	
	plugin_check(in, out);
	
	PLUGIN_ID("match.udp", "TCP matching routines");
	PLUGIN_VERSION(2, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");

	if ( !(matcher_add=in->import("matcher.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}
	
	if ( !(template_shortrange=in->import("template.shortrange")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=matcher_add("udp_sport", 12,
		udp_src_validate,
		in->import("template.cmp_sr"),
		in->import("template.ptr"));

	ok+=matcher_add("udp_dport", 13,
		udp_dst_validate,
		in->import("template.cmp_sr"),
		in->import("template.ptr"));
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
