/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*
*    This file represents the capdev subsystem. All capture
*    types are registered and found here.
*/
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>

#include <firestorm.h>
#include <rlim.h>
#include <packet.h>
#include <cleanup.h>
#include <capdev.h>

/* Packet serial number */
serial_t the_serial=0;

struct capdev *capdevs=NULL;
unsigned int num_capdevs=0;
unsigned int capdev_warned=0;

/* Fetch a new serial number */
void serial_number(serial_t *s)
{
	*s=the_serial++;
}


void capdev_cleanup(int code, void *priv)
{
	struct capdev *tmp, *foo;

	for(tmp=capdevs; tmp;) {
		foo=tmp;
		tmp=tmp->next;
		free(foo);
		foo=NULL;
	}
}

void capdev_init(void)
{
	cleanup_add(capdev_cleanup, NULL);
}

/* Find a capdev with the specified name */
struct capdev *capdev_find(const char *name)
{
	struct capdev *c;

	if ( !name ) return NULL;

	for(c=capdevs; c; c=c->next) {
		if ( !strcmp(c->name, name) ) return c;
	}

	return NULL;
}

/* plugins call this to register capdevs */
int capdev_add(const char *name,
	proc_capdev_init i,
	proc_capdev_end e,
	proc_capdev_go p)
{
	struct capdev *cap;

	if ( num_capdevs >= MAX_CAPDEVS ) {
		if ( !capdev_warned ) {
			capdev_warned++;
			mesg(M_WARN,"capdev: MAX_CAPDEVS exceeded.");
		}
		return 0;
	}

	if ( !name || !i || !e ) return 0;
	if ( capdev_find(name) ) {
		mesg(M_DEBUG,"capdev: error: '%s' capdev is "
			"already registered", name);
		return 0;
	}

	/* allocate the mem for the capdev struct */
	if ( !(cap=calloc(1, sizeof(struct capdev))) ) {
		cperror("calloc");
	}

	/* jot down the details */
	cap->name=name;
	cap->init=i;
	cap->end=e;
	cap->go=p;

	cap->next=capdevs;
	capdevs=cap;
	num_capdevs++;

	return 1;
}
